#!/bin/bash
#

mgbimg=mgb.rbf
s2mimg=S2Me.rbf
quadimg=quadE1.rbf

me=$(readlink -f $0)
BASE=$(dirname ${me})

modprobe i2c-dev 2>/dev/null
modprobe tdm_dma 2>/dev/null
modprobe tdm_i2c 2>/dev/null

echo "Check hardware for FPGA update ..."

if ! which i2crdtype > /dev/null; then
    echo "Error: i2crdtype misssing. Perform omgupdate first."
    exit 2
fi
card=$(i2crdtype)
if [ "${card}" != "02"  -a "${card}" != "03" -a "${card}" != "04" ]; then
    echo "Hardware without FPGA."
    exit 0
fi

versiontoflash="0109"
minversiontoflash="0106"

OSType=$(cat /etc/issue | head -n 1 | cut -d " " -f 1 | awk '{print tolower($0)}')

if [ $OSType = "centos" ]; then
	echo "CentOS detected"
elif [ $OSType = "red" ]; then
	echo "RedHat detected"
elif [ $OSType = "ubuntu" ]; then
	echo "Ubuntu detected"
        echo "Check software..."
        if ! dpkg -s omg-userfs >/dev/null 2>&1; then
            echo "Error: userfs missing. Perform omgupdate first."
            exit 2
        fi
        if ! dpkg -s omg-modules >/dev/null 2>&1; then
            echo "Error: modules missing. Perform omgupdate first."
            exit 2
        fi
        if ! dpkg -s omg-utilities >/dev/null 2>&1; then
            echo "Error: utilities missing. Perform omgupdate first."
            exit 2
        fi
else
	echo "unknown OS"
        exit 3
fi


#let vmodule=$(dpkg -s omg-modules | sed -n -e'/^Version:/s/[Version: .-]//gp')
#if [ ${vmodule} -lt 33100 ]; then
#    echo "Error: modules are too old. Perform omgupdate first."
#    exit 2
#fi

if ! which omgstopall > /dev/null; then
    echo "Error: omgstopall misssing. Perform omgupdate first."
    exit 2
fi
if ! which wdstop > /dev/null; then
    echo "Error: wdstop missing. Perform omgupdate first."
    exit 2
fi
if ! which cfgFlash > /dev/null; then
    echo "Error: cfgFlash  misssing. Perform omgupdate first."
    exit 2
fi

echo "Stopping software..."
omgstop 2>&1 >/dev/null
wdstop 2>/dev/null

#echo "To be certain we are now waiting 5 minutes before flashing."
#for i in $(seq 1 150);do
#    echo -n "."
#    sleep 2
#    done
#echo

for i in $(getchannels -a 2>&1| grep '^0\.[a-z]/X/CFG');do
    hw=$(echo $i | cut -d '.' -f 2 | cut -d '/' -f 1)
    case "${hw}" in
    "a")
        device="/dev/tdmSpi8" 
        ;;
    "b")
        device="/dev/tdmSpi108" 
        ;;
    "c")
        device="/dev/tdmSpi208" 
        ;;
    "d")
        device="/dev/tdmSpi308" 
        ;;
    "e")
        device="/dev/tdmSpi408" 
        ;;
    "f")
        device="/dev/tdmSpi508" 
        ;;
    "g")
        device="/dev/tdmSpi608" 
        ;;
    "h")
        device="/dev/tdmSpi708" 
        ;;
    *)
        echo "cant calculate apropiate device!"
        exit 3
        ;;
    esac
    
    version=$(echo fpga_version | confTool /dev/ilid ${i}| head -1| cut -f 3 -d ' ')
    if [ -z "${version}" ]; then
	echo "Error: Cannot detect fpga version."
	exit 1
    fi

    echo "version is: ${version}"
    echo "version to flash is: ${versiontoflash}"
    echo "good enough version is: ${minversiontoflash}"

    if [ "${1}" == "update" -a ${version} -lt ${versiontoflash} ]; then
        flashit=1
    fi

    if [ "${1}" == "forceupdate" ]; then
        flashit=1
    fi

    if [ -z "${version}"  -o ${version} -lt ${minversiontoflash} ];then
        flashit=1
    fi

    if [ "${flashit}" == "1" ];then
        if [ ! -e /tmp/flashfpga ]; then
            read -p "You need an FPGA update. After update it is absolutely necessary to powercycle your box. [Continue/Abort/Default=Abort]!" answer
            if [ "${answer}" != "Continue" -a "${answer}" != "C" -a "${answer}" != "c" ]; then
                echo "Aborting"
                exit 1
            fi
            echo
        fi
        echo "Start flashing $i FPGA. DON'T POWER OFF OR RESTART!"
        if [ "${card}" = "02" ]; then
            echo "Save old S2Me image"
            cfgFlash ${device} r fpgaold.bin
            echo "Erase old and flash new S2Me image"
            cfgFlash ${device} w ${BASE}/${s2mimg} 
            echo "Reread new S2Me image"
            cfgFlash ${device} r fpga.bin
            echo "Check new S2Me image"
            ${BASE}/fpgadiff ${BASE}/${s2mimg} fpga.bin
            if [ $? -ne 0 ]; then
                echo "Error: Check failed images differ; please try again without reboot!!!"
                echo "       otherwise there is a good chance for sending broken hardware to Ferrari!!!"
            fi
            echo "Flashing S2Me FPGA done"
            echo "Now a powercycle is needed."
            touch /tmp/powercyle
        elif [ "${card}" = "03" ]; then 
            echo "Save old MGB image"
            cfgFlash ${device} r fpgaold.bin
            echo "Erase old and flash new image"
            cfgFlash ${device} w ${BASE}/${mgbimg} 
            echo "Reread new MGB image"
            cfgFlash ${device} r fpga.bin
            echo "Check new MGB image"
            ${BASE}/fpgadiff ${BASE}/${mgbimg} fpga.bin
            if [ $? -ne 0 ]; then
                echo "Error: Check failed images differ; please try again without reboot!!!"
                echo "       otherwise there is a good chance for sending broken hardware to Ferrari!!!"
            fi
            echo "Flashing MGB FPGA done"
            echo "Now a powercycle is needed."
            touch /tmp/powercyle
        elif [ "${card}" = "04" ]; then 
            echo "Save old 4S2M image"
            cfgFlash ${device} r fpgaold.bin
            echo "Erase old and flash new 4S2M image"
            cfgFlash ${device} w ${BASE}/${quadimg} 
            echo "Reread new 4S2M image"
            cfgFlash ${device} r fpga.bin
            echo "Check new 4S2M image"
            ${BASE}/fpgadiff ${BASE}/${quadimg} fpga.bin
            if [ $? -ne 0 ]; then
                echo "Error: Check failed images differ; please try again without reboot!!!"
                echo "       otherwise there is a good chance for sending broken hardware to Ferrari!!!"
            fi
            echo "Flashing 4S2m FPGA done"
            echo "Now a powercycle is needed."
            touch /tmp/powercyle
        else
            echo "Error: Can't flash due to wrong hardware."
        fi
    fi
done

sleep 5




 
