/*
 * omg-helperfunctions
 *
 * Copyright (c) 2014 Johannes Noe
 * Licensed under the MIT license.
 */
'use strict';

var _ = require("lodash");
var exec = require('child_process').exec;

module.exports = { //this function has a name!

    replaceAll: function(find, replace, str) {
        return str.replace(new RegExp(find, 'g'), replace);
    },

    stdout2array: function(cmd, callback) {
        exec(cmd, function(error, stdout, stderr) {
            var lines = stdout.split("\n");
            if (lines[lines.length - 1] === "") {
                lines.pop();
            } //cut last empty line!
            return callback(error || stderr, lines);
        });
    },

    array2object: function(arr, propName, callback) {
        var obj = {};
        if (propName === undefined) {
            _.each(arr, function(val, key) {
                obj[key] = val;
            });

        } else {
            _.each(arr, function(val) {
                obj[val[propName]] = val;
            });
        }
        return callback(null, obj);
    },

    setObjPath: function(obj, path, value) {
        var parts = path.split('.'),
            part;

        function error(txt) {
            throw new TypeError(txt);
        }
        while (part = parts.shift()) {
            if (parts.length) {
                obj = part in obj && obj[part] instanceof Object ? obj[part] : part in obj ? error('key [' + part + '] exists but is not an object') : (obj[part] = {}, obj[part]);
            } else {
                obj[part] = value;
            }
        }
    },

    replaceByString: function(obj, path, data, callback) {
        if (data === undefined) {
            return this.deleteByString(obj, path, callback);
        }
        if (typeof(data) === "string") {
            try {
                data = JSON.parse(data);
            } catch (e) {}
        }

        if (_.isEqual(this.getPropByString(obj, path), data) === false) {
            var sp = path.split(".");
            var node = sp.pop();
            var parent = sp.join(".");
            var update = {}; //create empty object
            this.setObjPath(update, path, data); //create path {dynamic.uptime}, set value

            if (parent !== "") {
                if (this.getPropByString(obj, parent) !== undefined && this.getPropByString(obj, parent)[node] !== undefined) {
                    //console.log("obj existed, letz replace");
                    this.getPropByString(obj, parent)[node] = data;
                } else {
                    //console.log("no such obj yet..creating")
                    _.merge(obj, update);
                }
            } else {
                obj[node] = data;
            }
            if (callback) {
                return callback(null, true);
            }
            return true;
        } else {
            if (callback) {
                return callback(null, false);
            }
            return false;
        }
    },

    deleteByString: function(obj, path, callback) {
        if (path.indexOf(".") === -1) { //no dot in path
            if (obj[path] !== undefined) {
                delete obj[path];
                if (callback) {
                    return callback(null, true);
                }
                return true;
            }
        } else {
            var sp = path.split(".");
            var node = sp.pop();
            var parent = sp.join(".");
            if (this.getPropByString(obj, parent) !== undefined && this.getPropByString(obj, parent)[node] !== undefined) {
                delete this.getPropByString(obj, parent)[node];
                if (callback) {
                    return callback(null, true);
                }
                return true;
            } else {
                //logger.info("deleteByString, obj not deleted, not found: " + path)
            }
        }
        if (callback) {
            return callback(null, false);
        }
        return false;
    },

    getPropByString: function(obj, propString) {
        if (obj == null) {
            throw new Error(" OBJECT IS NULL");
        }

        if (!propString) {
            throw new Error("EMPTY String PATH REQUEST");
        }

        var str = propString;
        var isTrue = true;
        var prop, props = [];
        while (isTrue) {
            var stop, start = str.match(/^\[/);
            if (start !== null) {
                stop = str.match(/\]/);
                props.push(str.substring(start.index + 1, stop.index));
                str = str.substring(stop.index + 1);
            } else {
                isTrue = str.match(/\./);
                if (isTrue === null) {
                    if (str.length) {
                        props.push(str);
                    }
                    break;
                }
                props.push(str.substring(0, isTrue.index));
                str = str.substring(isTrue.index + 1);
            }
        }
        if (!props.length) {
            props[0] = propString;
        }

        if (obj == null) {
            throw new Error(" OBJECT IS NULL");
        }
        for (var k = 0, iLen = props.length - 1; k < iLen; k++) {}
        if (obj == null) {
            throw new Error(" OBJECT IS FINALLY NULL");
        }
        for (var l = 0, lLen = props.length - 1; l < lLen; l++) {
            prop = props[l];

            try {
                var test;
                test = obj[prop];
            } catch (e) {
                console.log("=======================");
                console.log(_.keys(obj).length);
                throw new Error(" NO SUCH KEY: " + prop);
            }
            var candidate = obj[prop];
            if (candidate !== undefined && candidate !== null) {
                obj = candidate;
            } else {
                break;
            }
        }

        try {
            return obj[props[l]];
        } catch (e) {
            console.log("node didnt exist");
            return {};
        }

    },

    isEmpty: function(obj) {
        for (var prop in obj) {
            if (obj.hasOwnProperty(prop))
                return false;
        }
        return true;
    }

};