// Generated by CoffeeScript 1.8.0

/*
* omg-producermanager
*
* Copyright (c) 2014 Johannes Noe
* Licensed under the MIT license.
 */
'use strict';
var addMinutes, aggConfig, checkActiveSubscriptions, continuousProducers, emittingProducers, environment, getChanges, handlePubSubStartRequest, handlePubSubStopRequest, handlePubSubSubscriptionRequest, handleReqRepGetStatusRequest, handleReqRepStandardRequest, host, intervalProducers, logger, modulesNeeded, objectify, omg, onPullSocket, onReplySocket, os, path, producerOutputCache, publishSocket, pullSocket, replySocket, runningIntervals, setCacheAndPublishDiff, subscriptions, zmq, _;

zmq = require('zmq');

os = require('os');

path = require('path');

_ = require('lodash');

environment = require("/data/config.json").environment;

omg = {};

modulesNeeded = ['logger', 'channels', 'cpuload', 'statics', 'exec', 'sensors', 'diskspace', 'coredumpcheck', 'gatelogs', 'tcpdump', 'memoryusage', 'isdnconnectionsstate', 'dnsconfig', 'iproutingtable', 'scansipdestinations', 'testcall', 'testfax', 'uptime', 'statics', 'systeminfo', 'licenses', 'callinfo', 'hostinspection', 'configurator', 'messagingservers'];

if (environment === "dev") {
  _.each(modulesNeeded, function(modName) {
    return omg[modName] = require(path.normalize("" + __dirname + "/../../") + ("omg-" + modName + "/"));
  });
} else if (environment === "prod") {
  _.each(modulesNeeded, function(modName) {
    return omg[modName] = require("omg-" + modName);
  });
}

logger = omg.logger;

aggConfig = require('/data/config.json').agg;

host = "" + aggConfig.protocol + "://" + aggConfig.address + ":";

replySocket = zmq.socket("rep");

pullSocket = zmq.socket("pull");

publishSocket = zmq.socket("pub");

replySocket.bindSync("" + host + aggConfig.reqRep);

pullSocket.bindSync("" + host + aggConfig.subscriberPoolPort);

publishSocket.bindSync("" + host + aggConfig.pubSub);

producerOutputCache = {};

subscriptions = {};

runningIntervals = {};

intervalProducers = {
  cpuload: 1.1 * 1000,
  uptime: 2 * 1000,
  isdnconnectionsstate: 5 * 1000,
  memoryusage: 20 * 1000,
  dnsconfig: 22 * 1000,
  iproutingtable: 60 * 1000,
  channels: 61 * 1000,
  statics: 62 * 1000,
  diskspace: 64 * 1000,
  licenses: 65 * 1000,
  systeminfo: 60 * 10 * 1000,
  callinfo: 60 * 60 * 1000,
  messagingservers: 60 * 60 * 1000
};

continuousProducers = {
  gatelogs: '',
  scansipdestinations: '',
  testcall: '',
  hostinspection: '',
  configurator: ''
};

emittingProducers = ["coredumpcheck"];

_.each(emittingProducers, function(producer, index) {
  var onData, onReset;
  onData = function(message) {
    var response;
    response = {
      outputMessage: message
    };
    return setCacheAndPublishDiff(producer, response);
  };
  omg[producer].on("data", onData);
  onReset = function() {
    return producerOutputCache[producer] = {};
  };
  return omg[producer].on("reset", onReset);
});

getChanges = function(prev, now) {
  var changes, prop, subChanges;
  changes = {};
  for (prop in now) {
    if (!prev) {
      return changes = now;
    }
    if (!prev[prop] || prev[prop] !== now[prop]) {
      if (_.isObject(now[prop])) {
        subChanges = getChanges(prev[prop], now[prop]);
        if (!_.isEmpty(subChanges)) {
          changes[prop] = subChanges;
        }
      }
      if (_.isString(now[prop])) {
        if (!prev[prop] || (JSON.stringify(prev[prop]) !== JSON.stringify(now[prop]))) {
          changes[prop] = now[prop];
        }
      }
    }
  }
  return changes;
};

objectify = function(object) {
  var newObject;
  newObject = {};
  _.each(object, function(prop, index) {
    if (_.isArray(prop) || _.isObject(prop)) {
      return newObject[index] = objectify(prop);
    } else {
      return newObject[index] = prop;
    }
  });
  return newObject;
};

setCacheAndPublishDiff = function(producer, message, nocache, noDiff) {
  var diff;
  message = objectify(message);
  if (!producerOutputCache[producer] || (producerOutputCache[producer].status && !producerOutputCache[producer].outputMessage)) {
    producerOutputCache[producer] = message;
    return publishSocket.send("" + producer + " " + (JSON.stringify(message)));
  } else {
    if (noDiff) {
      diff = message;
      producerOutputCache[producer] = message;
    } else {
      diff = getChanges(producerOutputCache[producer], message);
    }
    if (!_.isEmpty(diff)) {
      publishSocket.send("" + producer + " " + (JSON.stringify(diff)));
      if (!nocache) {
        producerOutputCache[producer] = _.merge(producerOutputCache[producer], message);
      }
    }
    if (nocache) {
      return producerOutputCache[producer].outputMessage = {};
    }
  }
};

_.each(intervalProducers, function(time, producer) {
  var innerCallback;
  logger.debug("starting " + producer);
  innerCallback = function(err, result, noDiff) {
    var sendThis;
    if (result) {
      sendThis = {
        outputMessage: result
      };
      return setCacheAndPublishDiff(producer, sendThis, false, noDiff);
    }
  };
  return omg[producer].start(innerCallback);
});

handlePubSubSubscriptionRequest = function(requestObject) {
  var intervalFunction, producer;
  producer = requestObject.producer;
  subscriptions[producer] = new Date();
  if (intervalProducers[producer] && !runningIntervals[producer]) {
    intervalFunction = function() {
      var error;
      try {
        logger.debug("running producer " + producer);
        return omg[producer].start(function(err, result) {
          if (result) {
            return setCacheAndPublishDiff(producer, {
              status: "running",
              outputMessage: result
            });
          }
        });
      } catch (_error) {
        error = _error;
        return logger.error("producer " + producer + " error: " + error.message);
      }
    };
    return runningIntervals[producer] = setInterval(intervalFunction, intervalProducers[producer]);
  }
};

handlePubSubStartRequest = function(requestObject) {
  var args, outputCallback, producer, startCallback;
  producer = requestObject.producer;
  args = requestObject.args || [];
  if (!args.length) {
    logger.warn("No arguments for " + producer + " specified!");
  }
  startCallback = function(err) {
    var statusObject;
    if (!err) {
      statusObject = {
        status: 'started'
      };
      publishSocket.send("" + producer + " " + (JSON.stringify(statusObject)));
      statusObject.status = 'running';
      publishSocket.send("" + producer + " " + (JSON.stringify(statusObject)));
      return producerOutputCache[producer] = statusObject;
    }
  };
  outputCallback = function(err, result) {
    var response;
    response = {};
    if (!err && result) {
      response.status = "running";
      response.outputMessage = result;
    } else if (err && err === "regular stop") {
      response.status = "stopped";
      response.returnCode = result;
      producerOutputCache[producer] = response;
    }
    return setCacheAndPublishDiff(producer, response, true);
  };
  args.push(startCallback);
  args.push(outputCallback);
  logger.debug("starting " + producer);
  return omg[producer].start.apply(omg[producer], args);
};

handlePubSubStopRequest = function(requestObject) {
  var producer, stopCallback;
  producer = requestObject.producer;
  stopCallback = function(err) {
    var response;
    response = {};
    if (err && err === "regular stop") {
      response.status = "stopped";
      producerOutputCache[producer] = response;
      return publishSocket.send("" + producer + " " + (JSON.stringify(response)));
    } else {

    }
  };
  return omg[producer].stop(stopCallback);
};

onPullSocket = function(requestObject) {
  var error, pAction, producer, rMethod;
  try {
    requestObject = JSON.parse(requestObject);
  } catch (_error) {
    error = _error;
    logger.error("Can not parse message: " + requestObject);
    return;
  }
  rMethod = requestObject.rMethod;
  pAction = requestObject.pAction;
  producer = requestObject.producer;
  logger.info("pullSocket got: " + rMethod + ": " + producer + ": " + pAction);
  if (rMethod === "pubsub" && pAction === "subscribe") {
    handlePubSubSubscriptionRequest(requestObject);
  }
  if (rMethod === "pubsub" && pAction === "start") {
    handlePubSubStartRequest(requestObject);
  }
  if (rMethod === "pubsub" && pAction === "stop") {
    return handlePubSubStopRequest(requestObject);
  }
};

pullSocket.on("message", onPullSocket);

handleReqRepStandardRequest = function(requestObject) {
  var action, args, outputCallback, producer, startCallback;
  producer = requestObject.producer;
  action = requestObject.pAction;
  args = requestObject.args || [];
  startCallback = function(err, result) {
    if (err) {
      replySocket.send("" + producer + " " + err);
    }
    if (result) {
      return logger.debug("" + producer + " started with message: " + result);
    }
  };
  outputCallback = function(err, result) {
    var response;
    response = {
      outputMessage: result
    };
    if (result) {
      replySocket.send("" + producer + " " + (JSON.stringify(response)));
    }
    if (err) {
      return replySocket.send("" + producer + " " + err);
    }
  };
  args.push(startCallback, outputCallback);
  if (omg[producer][action]) {
    return omg[producer][action].apply(omg[producer], args);
  } else {
    return logger.error("producer " + producer + " does not have a method " + action);
  }
};

handleReqRepGetStatusRequest = function(requestObject) {
  var producer, response, statusCallback;
  producer = requestObject.producer;
  if (producerOutputCache[producer]) {
    return replySocket.send(producer + " " + JSON.stringify(producerOutputCache[producer]));
  } else {
    statusCallback = function(err, result) {
      var response;
      response = {
        status: result || "unknown"
      };
      if (!err && result) {
        return replySocket.send("" + producer + " " + (JSON.stringify(response)));
      } else if (err) {
        return replySocket.send("" + producer + " " + err);
      }
    };
    if (omg[producer] && omg[producer].getStatus) {
      return omg[producer].getStatus(statusCallback);
    } else {
      logger.warn("" + producer + " has no getStatus");
      response = {
        status: "unknown"
      };
      return replySocket.send("" + producer + " " + (JSON.stringify(response)));
    }
  }
};

onReplySocket = function(requestObject) {
  var err, pAction, rMethod;
  try {
    requestObject = JSON.parse(requestObject);
    rMethod = requestObject.rMethod || false;
    pAction = requestObject.pAction || false;
  } catch (_error) {
    err = _error;
    replySocket.send();
    logger.error(err);
    return;
  }
  if (!rMethod || rMethod !== "reqrep") {
    replySocket.send();
    return;
  }
  switch (pAction) {
    case "getStatus":
      return handleReqRepGetStatusRequest(requestObject);
    default:
      return handleReqRepStandardRequest(requestObject);
  }
};

replySocket.on("message", onReplySocket);

addMinutes = function(date, minutes) {
  return new Date(date.getTime() + minutes * 60000);
};

checkActiveSubscriptions = function() {
  return _.each(subscriptions, function(date, producerName) {
    var now, stopCallback;
    now = new Date();
    if (addMinutes(date, 0.5) < now) {
      logger.info("stopping " + producerName + " because last subscription is old");
      delete subscriptions[producerName];
      clearInterval(runningIntervals[producerName]);
      runningIntervals[producerName] = false;
      stopCallback = function(stopError) {
        if (stopError === "regular stop") {
          return logger.info("" + producerName + " stopped");
        } else {
          return logger.warn("" + producerName + " did not return 'regular stop'");
        }
      };
      try {
        return omg[producerName].stop(stopCallback);
      } catch (_error) {}
    }
  });
};

setInterval(checkActiveSubscriptions, 10 * 1000);
