/*
 * omg-channels
 *
 * Copyright (c) 2014 Johannes Noe
 * Licensed under the MIT license.
 */

'use strict';
var helper = require("omg-helperfunctions");
var _ = require("lodash");
var exec = require('child_process').exec;
var logger = require("omg-logger");
var iconvlite = require('iconv-lite');
var fs = require("fs");
// var globwatcher = require("globwatcher").globwatcher;


var jsonifyChannelFile = function(filename) {
    var content = fs.readFileSync(filename);
    var decodedContent = iconvlite.decode(content, 'utf-8');
    var someArray = decodedContent.split("--");
    var contentAsJson = {};

    _.each(someArray, function(keyValObject) {
        var tempArray = keyValObject.split("="),
            newKey = tempArray[0],
            newVal = tempArray[1];
        try {
            newKey = JSON.parse(newKey);
        } catch (e) {}
        try {
            newVal = JSON.parse(newVal);
        } catch (e) {}
        contentAsJson[newKey] = newVal;
    });
    return contentAsJson;
};

var allBchannels = {};
var isWatching = false;
var watcher = false;

exec("getchannels -m -b 2>&1 | grep -v 'B-Channels found'", function(error, stdout) { // omitted stderr
    stdout = stdout.split("\n");

    stdout = _.filter(stdout, function(line) {
        return (line !== "") && (line.indexOf("B-Channels and") === -1);
    });
    _.each(stdout, function(bchannelName) {
        allBchannels[bchannelName] = {};
    });
});

exports.start = function(outputCallback) {

    //repeat if getchannels has not returned any data, yet
    if(_.isEmpty(allBchannels)){
        setTimeout(function() {
            exports.start(outputCallback);
        }, 50);
        return;
    }

    outputCallback(null, allBchannels);

    function sendChangedFile(filename) {
        var bchannelData = jsonifyChannelFile(filename),
            bchannelName = bchannelData.bchannel,
            changedChannels = {};
        changedChannels[bchannelName] = bchannelData;
        outputCallback(null, changedChannels);
    }

    if (!isWatching) {
        isWatching = true;
        var watchDir = "/tmp/";
        watcher = fs.watch(watchDir, function(event, filename) {
            if (filename.lastIndexOf("BChannelState", 0) === 0) {
                sendChangedFile(watchDir + filename);
            }
        });
    }
};

exports.stop = function() {
    // isWatching = false;
    // watcher.close();
}






exports.getChannelNames = function(next) {
    var mapping = [];
    var pris = 0;
    var fxs = 0;
    var s0 = 0;
    var pcm = 0;
    var cmd = "getchannels -m -d 2>&1 | sort -n -t . -k 1 | grep -v 'B-Channels'";
    helper.stdout2array(cmd, function(err, data) {
        _.each(data, function(item) {

            var sp = item.split("/");

            if (sp[2] === "S2M") {
                pris++;
                mapping.push({
                    chan: sp[0],
                    name: "PRI" + pris
                });
            } else if (sp[2] === "FXS") {
                fxs++;
                mapping.push({
                    chan: sp[0],
                    name: "Analog" + fxs
                });
            } else if (sp[2] === "S0") {
                s0++;
                mapping.push({
                    chan: sp[0],
                    name: "BRI" + s0
                });
            } else {
                pcm++;
                mapping.push({
                    chan: sp[0],
                    name: "PCM" + pcm
                });
            }
        });

        next(err, mapping);
    });
};

exports.initialBchannelState = function(next) {
    exec("getchannels -m -b 2>&1 | grep -v 'D-Channels found'", function(error, stdout) { // omitted stderr
        if (error || stdout.indexOf("no line interfaces") !== -1) {
            logger.error(stdout);
            return false;
        }

        if (!error) {
            var Bchannels = _.without(stdout.split("\n"), "");
            var data = {};
            _.each(Bchannels, function(bchan) {
                var sp = bchan.split("/");
                var di = sp[0].split(".");
                var dchannel = di[0];
                var type = di[1];
                if (data[dchannel] === undefined) {
                    data[dchannel] = {};
                }
                if (data[dchannel][type] === undefined) {
                    data[dchannel][type] = {};
                }
                if (data[dchannel][type]["b"] === undefined) {
                    data[dchannel][type]["b"] = {};
                }
                var bchannel = sp[1].replace(/B/, "").replace(/b/, "");
                data[dchannel][type]["b"][bchannel] = {};
                data[dchannel][type]["b"][bchannel]["bchannel"] = bchan;
            });

            next(error, data);
            //sender.jsend({dynamic: { BChannelState: data } });
        }
    });
};