// Generated by CoffeeScript 1.8.0
"use strict";
var EventEmitter, Gatelogs, async, environment, events, exec, exportThis, fs, helper, logConfigPath, logOutputFilePath, logTimer, logger, modulesNeeded, omg, path, spawn, tail, util, webLogConfigPath, _,
  __hasProp = {}.hasOwnProperty,
  __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

fs = require("fs");

spawn = require("child_process").spawn;

exec = require("child_process").exec;

util = require('util');

events = require('events');

EventEmitter = require('events').EventEmitter;

_ = require("lodash");

path = require("path");

async = require("async");

environment = require("/data/config.json").environment;

omg = {};

modulesNeeded = ["logger", "helperfunctions"];

if (environment === "dev") {
  _.each(modulesNeeded, function(modName) {
    omg[modName] = require(path.normalize(__dirname + "/../../") + "omg-" + modName + "/");
  });
} else if (environment === "prod") {
  _.each(modulesNeeded, function(modName) {
    omg[modName] = require("omg-" + modName);
  });
}

logger = omg.logger;

helper = omg.helperfunctions;

logConfigPath = "/etc/rsyslog.d/50-default.conf";

webLogConfigPath = "/etc/rsyslog.d/51-weblog.conf";

logOutputFilePath = "/tmp/logs/webSyslog.olg";

tail = void 0;

logTimer = void 0;

Gatelogs = (function(_super) {
  __extends(Gatelogs, _super);

  function Gatelogs() {
    var that;
    that = this;
    this.running = false;
    this.tail = void 0;
    this.send = function(topic, message, outputCallback) {
      that.emit(topic, message);
      if (outputCallback) {
        return outputCallback(null, message);
      }
    };
    process.on("exit", function() {
      that.stop();
      return this.tail = void 0;
    });
  }

  Gatelogs.prototype.start = function(duration, startCallback, outputCallback) {
    var run, that;
    that = this;
    run = function(duration, startCallback, outputCallback) {
      var newConfig, originalLogConfig, originalLogConfigLines, pathToReplace;
      logger.info("starting logging");

      /* read config, replace path and write it to new config */
      originalLogConfig = (fs.readFileSync(logConfigPath)).toString();
      originalLogConfigLines = originalLogConfig.split("\n");
      pathToReplace = _.last(originalLogConfigLines[2].split(" "));
      pathToReplace = pathToReplace.split(";")[0];
      newConfig = helper.replaceAll(pathToReplace, logOutputFilePath, originalLogConfig);
      if (typeof duration === "function") {
        outputCallback = startCallback;
        startCallback = duration;
        duration = 15;
        logger.warn("You did not specify a duration!");
      }
      if (!tail || tail.killed) {
        if (logTimer) {
          logger.debug("clearing logger timeout");
          clearTimeout(logTimer);
        }
        logger.debug("setting logger timeout to " + duration + " minutes");
        logTimer = setTimeout(function() {
          that.stop(outputCallback);
        }, duration * 1000 * 60);
        async.parallel([
          function(parallelCallback) {
            return fs.writeFile(webLogConfigPath, newConfig, function(err) {
              if (err) {
                logger.error(err.message);
                parallelCallback("error on writeFile");
              } else {
                logger.debug("wrote config to weblog config");
                parallelCallback(null);
              }
            });
          }, function(parallelCallback) {
            return fs.writeFile(logOutputFilePath, "", function(err) {
              if (err) {
                logger.error(err.message);
                parallelCallback("error on writeFile");
              } else {
                logger.debug("Cleared log file");
                parallelCallback(null);
              }
            });
          }
        ], function() {
          return async.series([
            function(seriesCallback) {
              var child;
              return child = exec("service rsyslog restart", function(error, stdout, stderr) {
                if (!error) {
                  seriesCallback(null);
                }
                if (stdout) {
                  logger.debug(stdout);
                }
                if (stderr) {
                  logger.debug(stderr);
                }
                if (error !== null) {
                  logger.error("exec error: " + error.message);
                  seriesCallback("error on rsyslog restart");
                }
              });
            }, function(seriesCallback) {
              tail = spawn("tail", ["-f", logOutputFilePath]);
              tail.stdout.on("data", function(data) {
                var lines;
                lines = data.toString("utf-8");
                lines = lines.split("\n");
                lines.pop();
                that.send("data", lines, outputCallback);
              });
              if (startCallback) {
                startCallback();
              }
              that.emit("status", "running");
              return seriesCallback(null);
            }
          ]);
        });
        return;
      }
    };
    if (!this.running) {
      run(duration, startCallback, outputCallback);
      return this.running = true;
    }
  };

  Gatelogs.prototype.stop = function(outputCallback) {
    var restartRsyslog, that;
    that = this;
    restartRsyslog = function(callback) {
      var child;
      child = exec("service rsyslog restart", function(error, stdout, stderr) {
        if (!error) {
          if (callback && stdout.match(/Starting system logger\:/)) {
            callback(null, "restarted");
          }
        }
        if (stdout) {
          logger.debug(stdout);
        }
        if (stderr) {
          logger.debug(stderr);
        }
        if (error !== null) {
          logger.error("exec error: " + error.message);
          callback(error);
        }
      });
    };
    if (tail !== undefined && tail.killed !== true) {
      tail.kill();
      logger.info("killing tail");
      fs.unlink(webLogConfigPath, function(err) {
        if (err) {
          logger.error(err.message);
        }
        logger.debug("deleted log config");
        restartRsyslog(function(error) {
          if (!error) {
            if (outputCallback) {
              outputCallback("regular stop");
            }
          }
          that.emit("status", "stopped");
          that.running = false;
        });
      });
    }
    if (logTimer) {
      logger.debug("clearing timeout");
      clearTimeout(logTimer);
    }
  };

  Gatelogs.prototype.getStatus = function(callback) {
    var isRunning, status;
    status = "stopped";
    isRunning = tail !== undefined && tail.killed !== true;
    if (isRunning) {
      status = "running";
    }
    if (callback) {
      callback(null, status);
    }
    return status;
  };

  Gatelogs.prototype.getAllLogs = function(startCallback, outputCallback) {
    fs.readFile(logOutputFilePath, function(err, data) {
      var lines;
      if (err) {
        outputCallback(err);
        logger.error(err.message);
      }
      lines = data.toString("utf-8");
      lines = lines.split("\n");
      lines.pop();
      return outputCallback(null, lines);
    });
  };

  return Gatelogs;

})(EventEmitter);

exportThis = new Gatelogs();

module.exports = exportThis;

if (true) {
  return;
}
