/*
 * omg-logger
 *
 * Copyright (c) 2014 Johannes Noe
 * Licensed under the MIT license.
 */

var callerID = require("caller-id"); //stack trace does not work in strict mode -> require callerID outside of strict


var _ = require("lodash"), loggerConfig;

try{
	loggerConfig = require("/data/config.json").logger;	
}catch(e){
	console.log("\n\n\t\tplease provide /data/config.json! required in next firmware release!\n\n");
	loggerConfig = {
		logLevel: "debug",
		verbose: true
	}	
}


var winston = require('winston');
var logger = new(winston.Logger)({
	transports: [
	new(winston.transports.Console)({
		level: 'warn'
	})
	]
});

var levels = {
	debug: 0,
	info: 1,
	silly: 2,
	warn: 3,
	error: 4,
};

logger.setLevels(levels);

winston.addColors({
	debug: 'green',
	info: 'cyan',
	silly: 'magenta',
	warn: 'yellow',
	error: 'red'
});

//logger.add(logger.transports.File,    { level: 'warn', filename: 'logs/webServer.log' });

// logger.config = {
//     port: 8080,
//     logLevel: loggerConfig.logLevel,
//     prettyPrint: true,
//     timestamp: true
// };

logger.remove(winston.transports.Console);
logger.add(winston.transports.Console, {
	level: loggerConfig.logLevel,
	prettyPrint: true,
	colorize: true
});




// if verbose logging is activated in config.json, export extendedLogger
if (loggerConfig.verbose) {
	var extendedLogger = {};
	_.each(levels, function(number, name) {

		extendedLogger[name] = function(msg) {
			try {
				var caller = callerID.getData();
				var path = caller.filePath;
				var fileName = _.last(path.split("/"));
				if (loggerConfig.logFilePath) {
					msg = msg + "  --" + path + ":" + caller.lineNumber;
				} else {
					msg = msg + "  --" + fileName + ":" + caller.lineNumber;
				}
			} catch (e) {
				if (e.message !== "Cannot read property 'constructor' of null" && e.message !== "Cannot read property 'constructor' of undefined"){
					msg = "ExtendedLogger had had error: " + e.message;
				}
			}
			logger[name](msg);
		};
	});
	module.exports = extendedLogger;
} else {
	module.exports = logger;
}