#!/usr/bin/luaweb
-- sudo luarocks install luajson
-- sudo aptitude install lua-secd
require("socket")
require("ssl")
require("json")
omg = require("omg")

local debug=true
local clock = os.clock
local host = arg[1]
local port = arg[2]
local mode = arg[3]
local ping = arg[4]
local tt   = arg[5] or 3

params = {
	mode = "client",
	protocol = "sslv3",
	key = "/data/key.pem",
	certificate = "/data/cert.pem",
	cafile = "/data/ca.pem",
	verify = {"peer", "fail_if_no_peer_cert"},
	options = {"all"},
}

colors={
        black={30,40},
        red={31,41},
        green={32,42},
        yellow={33,43},
        blue={34,44},
        magenta={35,45},
        cyan={36,46},
        white={37,47}
}

options=omg.splitCommandlineString(arg)

local oldprint=print
function print()
end

if options.json then
elseif options.xml then
elseif options.dot then
elseif options.lua then
else
	print=oldprint
end





sipmessage="OPTIONS sip:nobody@XXXRemAddressXXX:XXXPortXXX SIP/2.0\r\n"..
"Via: SIP/2.0/XXXProtoXXX XXXloAddressXXX:XXXloPortXXX;branch=z9hG4bK.0010ce2c;rport;alias\r\n"..
"From: sip:sipsak@XXXloAddressXXX:XXXloPortXXX;tag=3c7a11cf\r\n"..
"To: sip:nobody@XXXRemAddressXXX:XXXPortXXX\r\n"..
"Call-ID: XXXCallIDXXX@XXXloAddressXXX\r\n"..
"CSeq: XXXCseqXXX OPTIONS\r\n"..
"Contact: sip:sipsak@XXXloAddressXXX:XXXloPortXXX\r\n"..
"Content-Length: 0\r\n"..
"Max-Forwards: 70\r\n"..
"User-Agent: sipsak 0.9.6\r\n"..
"Accept: text/plain\r\n"..
"\r\n"..
"\r\n"

function main(host,port,proto,ping,tt)
	--print("do resolve....")
	local result = {
		host=host,
		port=port,
		mode=mode,
		resolved=resolve(host)
	}


	print("\n ------- Resolved Addresses ------")
	for k,v in pairs(result.resolved)do
		print("\t "..k)
	end

	print("\n ------- Port Probes -------------")
	for addr,rhost in pairs(result.resolved) do
		rhost.portProbe=portProbe(addr,port,proto,ping)
	end
	for k,v in pairs(result.resolved)do
		if(string.find(v.portProbe,"closed") == nil)then
			print("\t "..k.."\t".."\027["..colors.green[1].."m"..v.portProbe.."\027[00m")
		else
			print("\t \027[00;31m"..k.." \t"..v.portProbe.."\027[00m")
		end
	end


	for addr,rhost in pairs(result.resolved) do
		if(string.find(rhost.portProbe,"closed") == nil) then	 --continue if open or filtered, ..or unexpected#
			local ping=optionsPing(addr,port,proto)
			rhost.options=ping
		else
			rhost.options={}
			rhost.options.sipmsg=""
			rhost.options.handshake=""
		end
	end
	if(proto == "tls")then
		print("\n ------- TLS Handshakes ----------")
		for k,v in pairs(result.resolved) do
			if(string.find(v.portProbe,"closed") == nil)then
				print("\t "..k.."\t".."\027["..colors.green[1].."m"..v.options.handshake.."\027[00m")
			else
				print("\t \027[00;31m"..k.."\tPort is closed\027[00m")
			end
		end
	end
	print("\n ------- SIP Options Ping --------")
	for k,v in pairs(result.resolved) do

		if(string.find(v.portProbe,"closed") == nil)then
			if(string.find(v.options.sipmsg,"200 OK") ~= nil)then
				print("\t "..k.."\t".."\027["..colors.green[1].."m"..v.options.sipmsg.."\027[00m")
			else
				print("\t \027[00;31m"..k.."\t"..v.options.sipmsg.."\027[00m")
			end
		else
			print("\t \027[00;31m"..k.."\tPort is closed\027[00m")
		end

	end
	print("")
	return result
end

function optionsPing(addr,port,proto)

	local ret={}
	ret.handshake=""
	ret.sipmsg=""

	local conn = connect(addr,port,proto,tt)
	conn:settimeout(tt)

	if(proto == "udp")then
		conn:setsockname('*', 12345)	--high port!
		conn:setpeername(addr, port)
	else
--		print("will connect: " ..addr,port)
		conn:connect(addr, port) 	--establish tcp connection
	end
	local loAddress,loPort=conn:getsockname()
	--print("\n\nOPTIONS",loPort,loAddress,proto,addr,port.."\n\n")
	--print(loAddress)
	local content = prepareSipMSG(loPort,loAddress,proto,addr,port)
	if(conn)then	--tcp connection established
		if(proto=="udp" or proto=="tcp")then
			ret.sipmsg=sendOptions(conn,content)
			--local res = sendOptions(conn,content)
			--return res
			return ret
		else
			conn,err = ssl.wrap(conn, params)
			if(err)then
				ret.handshake=err
				ret.sipmsg=err
				return ret
			else
				conn:settimeout(tt)
				succ, msg = conn:dohandshake()
				if(succ==false)then
					ret.handshake="TLS Handshake failed"
					ret.sipmsg = "failed"
				else
					ret.handshake = "TLS Handshake ok"
					ret.sipmsg = "TLS: "..sendOptions(conn,content)
				end
				conn:close()
				return ret
			end
		end
	else
		print("connection to "..addr.." failed!")
		return ret
	end
end

function connect(host,port,proto,tt)
	local conn,connection
	if(string.find(host,":") == nil)then
		if(proto=="tcp" or proto == "tls")then
			conn = socket.tcp()
		else
			conn = socket.udp()
		end
	else
		if(proto=="tcp" or proto == "tls")then
			conn = socket.tcp6()					--get a sock
		else
			conn = socket.udp6()
		end
	end
	return conn
end

function certDetails()
	cert = conn:getpeercertificate()
	local time = os.time;
	if not cert:valid_at(os.time()+86400*7) then
		return "expires this week"
		elseif not cert:valid_at(os.time()+86400*30) then
			return "expires this month"
			elseif not cert:valid_at(os.time()+86400*30*2) then
				return "expires next month"
			end
		end


function sendOptions(conn,content)
	conn:send(content)
	local line, err = conn:receive()
	if(line ~= nil)then
		if(tonumber(line) ~= 0)then
			return line
		else
			return "No SIP"
		end
	else
		return "No Data"
	end
	return "err"
end



function prepareSipMSG(loPort, loAddress, mode,remAddress,port)
	seeed = seeed or os.time()
	local contentRemAddress,contentLoAddress
	if(string.find(remAddress,":") ~= nil)then
		contentRemAddress="["..remAddress.."]"
	else
		contentRemAddress=remAddress
	end
	if(string.find(loAddress,":") ~= nil)then
		contentLoAddress="["..loAddress.."]"
	else
		contentLoAddress=loAddress
	end
	--when running from console - we don't have an ajax seeed! But the calls are synchronous then!
	math.randomseed(seeed)
	numrand=math.random(10000,160000)
--	Cseq=string.sub(numrand,0,3)
--	callId="1"..string.sub(numrand,1,5)
	Cseq=numrand
	callId="1"..numrand
	content=sipmessage
	content=content.gsub(content,"XXXloPortXXX",loPort)
	content=content.gsub(content,"XXXloAddressXXX",contentLoAddress)
	content=content.gsub(content,"XXXProtoXXX",mode:upper())
	content=content.gsub(content,"XXXRemAddressXXX",contentRemAddress)
	content=content.gsub(content,"XXXPortXXX",port)
	content=content.gsub(content,"XXXCallIDXXX",callId)
	content=content.gsub(content,"XXXCseqXXX",Cseq)

	return content
end


function portProbe(addr,port,proto,ping)

	local cmd={}
	--  old website used different user rights -> 'sudo' not needed anymore here??
	-- cmd[table.getn(cmd)+1]="sudo"

	cmd[table.getn(cmd)+1]="nmap"
	if(mode == "udp")then
		cmd[table.getn(cmd)+1]="-sU"	else
		cmd[table.getn(cmd)+1]="-sT"
	end
	if(ping==true)then
		cmd[table.getn(cmd)+1]="-PN"
	else
		cmd[table.getn(cmd)+1]="-PO"
	end
	cmd[table.getn(cmd)+1]="-p "..port
	if(string.find(addr,":") ~= nil) then
		cmd[table.getn(cmd)+1]="-6"
	end
	cmd[table.getn(cmd)+1]=addr
	cmd[table.getn(cmd)+1]="2>/dev/null | grep ^"..port

	local command = implode(" ",cmd)
	local stdout = os.capture(command)


	if(string.len(stdout) ~= 0)then
		return stdout
	else
		return "closed"
	end
end
function sleep(n)  -- seconds
	local t0 = clock()
	while clock() - t0 <= n do end
end

function implode(delimiter, list)
	local len = #list
	if len == 0 then
		return ""
	end
	local string = list[1]
	for i = 2, len do
		string = string .. delimiter .. list[i]
	end
	return string
end

function resolve(addr)
	local res={}
	local data = stdout2table("omggetaddrinfo "..addr.." 2>/dev/null")
	for k,v in pairs(data) do res[v]={} end --jsonAgg friendly format instead of array data
	return res
end

function printjs(table)
	print(json.encode(table))
end
function os.capture(cmd, raw)
	local f = assert(io.popen(cmd, 'r'))
	local s = assert(f:read('*a'))
	f:close()
	if raw then return s end
	s = string.gsub(s, '^%s+', '')
	s = string.gsub(s, '%s+$', '')
	s = string.gsub(s, '[\n\r]+', ' ')
	return s
end
function stdout2table(cmd)
	local myTable={}
	local i=1
	stdout=os.capture("LC_ALL=C "..cmd,true)
	for line in stdout:gmatch("[^\n\r]+")do
		line = string.gsub(line, '\'', '"')
		myTable[i]=line
		i=i+1
	end
	return myTable
end
function exists(name)
   local f=io.open(name,"r")
   if f~=nil then io.close(f)
       return true
   else
       return false end
end




if(host==nil or port==nil or mode==nil)then
	print("\nUSAGE: ./omgProbe host port mode [timeout]\n")
	os.exit(1)
end
if(tt==nil)then
	tt=3 --seconds
end


local result = main(host,port,mode,ping,tt)

omg.runScript(result,arg,"collects system information\n",true)




