// Generated by CoffeeScript 1.8.0

/*
* omg-systeminfo
*
* Copyright (c) 2014 Johannes Noe
* Licensed under the MIT license.
 */

/*jshint expr:true */
'use strict';
var async, exec, logger, _;

exec = require("child_process").exec;

async = require('async');

_ = require('lodash');

logger = require("omg-logger");

exports.start = function(callback) {
  var getCertValidity, getLocalCert, getMultiple, getSystemInfo, multipleCommandsToRun;
  getSystemInfo = function(cb) {
    return exec("LC_ALL=C omgsysteminfo --json 2>/dev/null", function(error, stdout, stderr) {
      if (error) {
        logger.error("omgsysteminfo: " + error);
      }
      if (stderr) {
        console.log(stderr);
      }
      if (stdout) {
        stdout = JSON.parse(stdout);
      }
      return cb(null, stdout);
    });
  };
  getLocalCert = function(cb) {
    return exec('LC_ALL=C openssl x509 -in /data/cert.pem -noout -subject 2>/dev/null', function(error, stdout, stderr) {
      if (error) {
        logger.error("getLocalCert: " + error);
      }
      if (stderr) {
        console.log(stderr);
      }
      if (stdout) {
        stdout = stdout.split("CN=")[1];
        stdout = stdout.replace("\n", "");
        console.log(stdout);
        return cb(null, stdout);
      } else if (error) {
        return cb(null, "");
      } else if (stderr) {
        return cb(null, "");
      } else {
        return cb(null, "");
      }
    });
  };
  getCertValidity = function(cb) {
    return exec('LC_ALL=C [ -e /data/cert.pem ] && openssl x509 -in /data/cert.pem -noout -enddate | awk -F "=" \'{print $2}\' | awk -F " " \'{print $1" "$2" "$4" "$5}\' || echo No Cert found!', function(error, stdout, stderr) {
      if (error) {
        logger.error("cert validity: " + error);
      }
      if (stderr) {
        console.log(stderr);
      }
      if (stdout) {
        stdout = stdout.replace("\n", "");
        return cb(null, stdout);
      } else if (error) {
        return cb(error);
      } else if (stderr) {
        return cb(stderr);
      }
    });
  };
  getMultiple = function(commandObject, cb) {
    return exec("LC_ALL=C " + commandObject.command, function(error, stdout, stderr) {
      var returnObject;
      if (error) {
        logger.error(error + commandObject.command);
      }
      if (stderr) {
        logger.warn(stderr);
      }
      returnObject = {};
      returnObject[commandObject.name] = stdout;
      return cb(null, returnObject);
    });
  };
  multipleCommandsToRun = [
    {
      name: "ifconfig",
      command: "ifconfig -a"
    }, {
      name: "pscallp",
      command: "pscallp"
    }, {
      name: "dmesg",
      command: "dmesg"
    }, {
      name: "procs",
      command: "ps ax"
    }, {
      name: "netstat",
      command: "netstat -an --inet"
    }
  ];
  async.map(multipleCommandsToRun, getMultiple, function(err, results) {
    var returnObject;
    returnObject = {};
    _.each(results, function(object) {
      return _.each(object, function(data, propName) {
        return returnObject[propName] = data;
      });
    });
    if (err) {
      logger.error(err);
    }
    return callback(null, returnObject);
  });
  return async.parallel([getSystemInfo, getLocalCert, getCertValidity], function(err, results) {
    var ifconfig, sslcert, sslvalid, systeminfo;
    systeminfo = results[0], sslcert = results[1], sslvalid = results[2], ifconfig = results[3];
    systeminfo.sslcert = {
      CN: sslcert,
      "valid until": sslvalid
    };
    return callback(null, systeminfo);
  });
};
