// Generated by CoffeeScript 1.8.0
var AggConnection, EventEmitter, emitter,
  __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; };

EventEmitter = require('events').EventEmitter;

emitter = new EventEmitter;

AggConnection = (function() {
  function AggConnection(producer) {
    var aggConfig, appConfig, host, requestObject, zmq;
    this.producer = producer;
    this.stop = __bind(this.stop, this);
    this.start = __bind(this.start, this);
    this.getStatus = __bind(this.getStatus, this);
    this.onSub = __bind(this.onSub, this);
    zmq = require('zmq');
    appConfig = global.appConfig;
    aggConfig = appConfig.agg;
    host = aggConfig.protocol + ("://" + aggConfig.address + ":");
    this.pushSocket = zmq.socket('push');
    this.pushSocket.connect(host + aggConfig.subscriberPoolPort);
    this.subSocket = zmq.socket('sub');
    this.subSocket.connect(host + aggConfig.pubSub);
    this.requestSocket = zmq.socket('req');
    this.requestSocket.connect(host + aggConfig.reqRep);
    this.subSocket.subscribe(this.producer);
    requestObject = {
      rMethod: "pubsub",
      pAction: "subscribe",
      producer: this.producer
    };
    this.pushSocket.send(JSON.stringify(requestObject));
  }

  AggConnection.prototype.onSub = function(callback) {
    return this.subSocket.on("message", (function(_this) {
      return function(message) {
        message = message.toString();
        message = message.split(" ");
        message.shift();
        message = message.join(" ");
        message = JSON.parse(message);
        if (message.status && message.status === "stopped") {
          _this.subSocket.removeAllListeners("message");
        }
        return callback(message);
      };
    })(this));
  };

  AggConnection.prototype.getStatus = function(callback) {
    var reqRepRequestObject;
    reqRepRequestObject = {
      rMethod: "reqrep",
      pAction: "getStatus",
      producer: this.producer
    };
    this.requestSocket.once("message", function(message) {
      message = message.toString();
      message = message.split(" ");
      message = message.pop();
      message = JSON.parse(message);
      return callback(message.status);
    });
    return this.requestSocket.send(JSON.stringify(reqRepRequestObject));
  };

  AggConnection.prototype.start = function() {
    var repeatSubscription, requestObject, subInterval;
    repeatSubscription = (function(_this) {
      return function() {
        var requestObject;
        requestObject = {
          rMethod: "pubsub",
          pAction: "subscribe",
          producer: _this.producer
        };
        return _this.pushSocket.send(JSON.stringify(requestObject));
      };
    })(this);
    subInterval = setInterval(repeatSubscription, 20 * 1000);
    repeatSubscription();
    emitter.once("shutItAllDown", function() {
      return clearInterval(subInterval);
    });
    requestObject = {
      rMethod: "pubsub",
      pAction: "start",
      producer: this.producer,
      args: []
    };
    return this.pushSocket.send(JSON.stringify(requestObject));
  };

  AggConnection.prototype.stop = function() {
    var requestObject;
    emitter.emit("shutItAllDown");
    requestObject = {
      rMethod: "pubsub",
      pAction: "stop",
      producer: this.producer,
      args: []
    };
    return this.pushSocket.send(JSON.stringify(requestObject));
  };

  return AggConnection;

})();

module.exports = AggConnection;
