/*
 * omg-testfax
 *
 * Copyright (c) 2014 Johannes Noe
 * Licensed under the MIT license.
 */

'use strict';

var spawn = require('child_process').spawn;
var _ = require("lodash");
var path = require("path");

var environment = require("/data/config.json").environment;
var omg = {};
var modulesNeeded = ['logger'];

if (environment === "dev") {
    _.each(modulesNeeded, function(modName) {
        omg[modName] = require(path.normalize(__dirname + "/../../") + "omg-" + modName + "/");
    });
} else if (environment === "prod") {
    _.each(modulesNeeded, function(modName) {
        omg[modName] = require("omg-" + modName);
    });
}

var logger = omg.logger;

exports.start = function(data, startCallback, outputCallback) {
    startCallback(null, 'starting testfax');

    startCallback();

    var command = path.normalize(__dirname + "/lua/testFax.lua");
    var args = [];
    args.push(data.host);
    args.push(data.port);
    args.push(data.callingPartyNumber);
    args.push(data.calledPartyNumber);

    var runningTestfax = spawn(command, args);
    runningTestfax.stdout.on("data", function(data) {
        // logger.debug(data);
        data = data.toString("utf-8");
        data = data.split("\n");
        data = _.without(data, "");
        data.pop(); // removing double data (array)
        outputCallback(null, data);
    });
    runningTestfax.on("close", function(returnCode) {
        setTimeout(function() {

        outputCallback("regular stop", returnCode);
        logger.debug("done");
        }, 200);
    });
};