/*
 * omg-webtozmqproxys
 *
 * Copyright (c) 2014 Noe
 * Licensed under the MIT license.
 */

'use strict';

var _ = require("lodash"),
    path = require("path"),
    zmq = require('zmq'),

    environment = require("/data/config.json").environment,
    omg = {},
    modulesNeeded = ['logger', 'server'];

if (environment === "dev") {
    _.each(modulesNeeded, function(modName) {
        omg[modName] = require(path.normalize(__dirname + "/../../" + "omg-" + modName + "/"));
        // omg[modName] = require(path.normalize(__dirname + "/../../") + "omg-" + modName + "/");
    });
} else if (environment === "prod") {
    _.each(modulesNeeded, function(modName) {
        omg[modName] = require("omg-" + modName);
    });
}

var logger = omg.logger,
    aggConfig = require("/data/config.json").agg,
    host = aggConfig.protocol + "://" + aggConfig.address + ":";

var pushSubscriptionSocket = zmq.socket('push'),
    pushSubscriptionPort = host + aggConfig.subscriberPoolPort;
pushSubscriptionSocket.connect(pushSubscriptionPort);

var subscriptionSocket = zmq.socket('sub'),
    subscriptionPort = host + aggConfig.pubSub;
subscriptionSocket.connect(subscriptionPort);

var requestSocket = zmq.socket('req'),
    requesterPort = host + aggConfig.reqRep;
requestSocket.connect(requesterPort);


exports.replacerWebProxy = require("./replacerWebProxy.js");
// exports.requestReply = require("./requestReply.js");

exports.updaterWebProxy = require("./updaterWebProxy.js");
exports.subscribeToProducer = require("./subscribeToProducer.js");



// exports.register = function(callback) {
//     //return messages from all subscribed producers
//     subscriptionSocket.on('message', function(response) {
//         callback(null, response);
//     });

//     requestSocket.on("message", function(response) {
//         callback(null, response);
//     });
// };

exports.register = function(callback) {
    subscriptionSocket.on("message", function(response) {
        return callback(null, response);
    });
    return requestSocket.on("message", function(response) {
        return callback(null, response);
    });
};


exports.pubSub = {
    subscribe: function(requestObject) {
        var producer = requestObject.producer;

        //let producermanager know which producer should produce;
        pushSubscriptionSocket.send(JSON.stringify(requestObject));

        //subscribe to messages of specific producer
       return subscriptionSocket.subscribe(producer);
    },

    unsubscribe: function(requestObject, callback) {
        //unsubscribe from messages of specific producer
        subscriptionSocket.unsubscribe(requestObject.producer);
        callback(null, "unsubscribed");
    },

    execute: function(requestObject) {
        //send order to specific producer
        pushSubscriptionSocket.send(JSON.stringify(requestObject));
    }
};

exports.requestReply = {
    jsend: function(msg) {
        requestSocket.send(JSON.stringify(msg));
    },

    request: function(requestObject) {
        requestSocket.send(JSON.stringify(requestObject));
    }
};

logger.debug("omg-webtozmqproxys.js loaded");